<?php

namespace EventPortal\Domain\Util\Notification;

use EventPortal\Domain\Event\Models\Event;
use Illuminate\Support\Collection;
use Illuminate\Support\Facades\Notification as LaravelNotification;

class Notification
{
    /** @var string */
    protected $key;

    /** @var callable|string */
    public $subject;

    /** @var NotificationData */
    public $data;

    /** @var \Illuminate\Support\Collection|array|mixed */
    public $recipients;

    /** @var string */
    public $from;

    /** @var string */
    public $replyTo;

    /** @var \Illuminate\Support\Collection */
    protected $callables;

    public function __construct(string $key)
    {
        $this->key = $key;
        $this->callables = new Collection();
        $this->data = new NotificationData();
    }

    /**
     * Set the subject, or register a callback to
     * set it later, once data is available.
     *
     * @param string|callable $subject
     */
    public function subject($subject): self
    {
        if (is_callable($subject)) {
            $this->callables->push('subject');
        }

        $this->subject = $subject;

        return $this;
    }

    public function event(Event $event): self
    {
        $this->data->setEvent($event);

        return $this;
    }

    public function form(array $form): self
    {
        $this->data->setForm($form);

        return $this;
    }

    /**
     * @param mixed $recipients
     */
    public function to($recipients): self
    {
        $this->recipients = $recipients;

        return $this;
    }

    public function from(string $sender): self
    {
        $this->from = $sender;

        return $this;
    }

    public function replyTo(string $replyTo): self
    {
        $this->replyTo = $replyTo;

        return $this;
    }

    public function send(): void
    {
        $this->resolveCallbacks();

        LaravelNotification::send($this->recipients, new SystemNotification(
            $this->key,
            $this->data,
            $this->subject,
            $this->from,
            $this->replyTo
        ));
    }

    public function toArray(): array
    {
        return [
            'subject' => $this->subject,
            'data' => $this->data->toArray(),
            'recipients' => $this->recipients,
            'from' => $this->from,
            'replyTo' => $this->replyTo,
        ];
    }

    private function resolveCallbacks(): void
    {
        $this->callables->each(function (string $property): void {
            if (is_callable($this->$property) !== true) {
                return;
            }
            $this->$property = call_user_func($this->$property, $this->data);
        });
    }
}
