<?php

namespace EventPortal\Http\Controllers\Delegate\Event\Form;

use EventPortal\Domain\Component\Button;
use EventPortal\Domain\Delegate\DelegateForm;
use EventPortal\Domain\Event\Form\Models\Form as FormModel;
use EventPortal\Domain\Event\Form\Models\FormStep;
use EventPortal\Domain\Event\Models\Event;
use EventPortal\Domain\Event\ValueObjects\EventDetails;
use EventPortal\Domain\Form\Facades\Form;
use EventPortal\Domain\Util\DelegateRender;
use EventPortal\Enums\InertiaMethod;
use Illuminate\Http\Request;
use Illuminate\Http\Response as HttpResponse;

class ViewFormController
{
    /**
     * @return \Illuminate\Http\RedirectResponse|\Inertia\Response
     */
    public function __invoke(Request $request, Event $event, FormModel $form)
    {
        $delegate = $request->user()->delegate;

        $delegateForm = DelegateForm::make($delegate, $form);

        if ($delegateForm->allStepsCompleted()) {
            toast()->success('You have already completed this form.');

            return redirect()->route('delegate.landing', ['event' => $event->slug]);
        }

        $step = $delegateForm->currentStep();

        if ($request->has('step')) {
            $step = FormStep::forForm($form)->findOrFail($request->step);

            abort_if(
                $delegate->formSteps()->where('form_step_id', $step->id)->doesntExist(),
                HttpResponse::HTTP_NOT_FOUND
            );
        }

        $formComponent = Form::makeFromDatabase($form, $step)
            ->model($delegate)
            ->context($event, $step)
            ->addActions(static function (Event $event) use ($form, $step, $delegateForm): array {
                if ($step->isFirst() === false) {
                    $buttons[] = Button::make('Back')
                        ->variant('secondary')
                        ->method(InertiaMethod::VISIT())
                        ->route('delegate.form.index', [
                            'event' => $event->slug,
                            'form' => $form->slug,
                            'step' => $delegateForm->previousStep($step),
                        ]);
                }

                $buttons[] = Button::make('Save for later')
                    ->variant('secondary')
                    ->method(InertiaMethod::POST())
                    ->route('delegate.form.update', [
                        'form' => $form->slug,
                        'event' => $event->slug,
                        'step' => $step,
                    ]);

                return $buttons;
            })
            ->addPrimaryAction(static function (Event $event) use ($form, $step): Button {
                return Button::make($step->isLast() ? 'Finish' : 'Next step')
                    ->method(InertiaMethod::POST())
                    ->route('delegate.form.store', [
                        'event' => $event->slug,
                        'form' => $form->slug,
                        'step' => $step,
                    ]);
            })
            ->render();

        return DelegateRender::component('Delegate/Form/Index')
            ->key('delegate.form.index')
            ->withForm($formComponent)
            ->withEventDetails(EventDetails::make($event))
            ->get();
    }
}
